import { staticImplements } from '../utils'
import { Deserialize, Deserializer, IterableAccess, MapAccess, Visitor } from './interface'

@staticImplements<Deserialize<any>>()
export class GenericSeed<T> implements Deserialize<T> {
  static deserialize<T, D extends Deserializer>(deserializer: D): T {
    return deserializer.deserializeAny(new GenericVisitor<T>())
  }

  deserialize<D extends Deserializer>(deserializer: D): T {
    return GenericSeed.deserialize(deserializer)
  }
}

export class GenericVisitor<T> implements Visitor<T> {
  visitString(value: string): T {
    return value as T
  }

  visitNumber(value: number): T {
    return value as T
  }

  visitBigInt(value: bigint): T {
    return value as T
  }

  visitBoolean(value: boolean): T {
    return value as T
  }

  visitSymbol(value: symbol): T {
    return value as T
  }

  visitNull(): T {
    return null as T
  }

  visitObject(access: MapAccess): T {
    const result: Record<PropertyKey, any> = {}
    let entry

    while ((entry = access.nextEntry<string, any>())) {
      result[entry[0]] = entry[1]
    }

    return result
  }

  visitFunction?(value: Function): T {
    return value as T
  }

  visitMap?(access: MapAccess): T {
    const result = new Map()
    let entry

    while ((entry = access.nextEntry<string, any>())) {
      result.set(entry[0], entry[1])
    }

    return result as T
  }

  visitIterable?(access: IterableAccess): T {
    const result = new Array(access.sizeHint())
    let element

    while ((element = access.nextElement())) {
      result.push(element)
    }

    return result as T
  }

  visitClass?(_name: string, _fields: string[], _value: any): T {
    throw new Error('Method not implemented.')
  }
}

