import { CaseConvention, Constructor, staticImplements } from '../utils'
import { GenericVisitor } from './generic'
import { Deserialize, Deserializer } from './interface'

export interface DeserializationOptions {
  rename?: string
  renameAll?: CaseConvention
}

const DefaultDeserializationOptions = {}

export function deserialize(options?: DeserializationOptions) {
  options = {
    ...DefaultDeserializationOptions,
    ...options
  }

  return function <T, C extends Constructor>(constructor: C) {
    @staticImplements<Deserialize<T>>()
    class Deserializable extends constructor {
      static deserialize<D extends Deserializer>(deserializer: D): T {
        const visitor = new GenericVisitor<T>()
        return deserializer.deserializeAny(visitor)
      }
    }

    return Deserializable
  }
}

