import { CaseConvention, Constructor } from '../utils'
import { serializeWith } from './impl'
import { isGenericSerializer, Serializer } from './interface'

export interface SerializationOptions {
  default?: <T>() => T
  rename?: string
  renameAll?: CaseConvention
  tag?: string
  untagged?: boolean
  withInherited?: boolean
}

const DefaultSerializationOptions: SerializationOptions = {
  withInherited: true
}

export function serialize(options?: SerializationOptions) {
  options = {
    ...DefaultSerializationOptions,
    ...options
  }

  return function <T extends Constructor>(constructor: T) {
    return class Serializable extends constructor implements Serializable {
      static name = constructor.name
      serialize<U>(serializer: Serializer<U>): U {
        // shortcut for serializers with only the serializeAny method
        if (isGenericSerializer(serializer)) {
          return serializer.serializeAny!(this) as U
        } else {
          return serializeWith(serializer, this) as U
        }
      }
    }
  }
}
